
#include<stdlib.h>
#include<iostream>
#include<string>
#include<vector>
#include<math.h>
using namespace std;

int main(int argc, char** argv) {
	int base = 2;
	int num = 3;
	string alphabet = "0123456789ABCDEF";
	int diameter = 120;
	int offset = 10;

	if(argc > 1) {
		num = atoi(argv[1]);
	} else {
		cerr << argv[0] << " ERROR: no coset size given\n";
		return 1;
	}
	if(num < 0) {
		cerr << argv[0] << " ERROR: " << argv[1] << " is not a coset size\n";
		return 1;
	}

	if(argc > 2) {
		base = atoi(argv[2]);
	}
	if(base < 0) {
		cerr << argv[0] << " ERROR: " << argv[2] << " is not a legal base of a numbering system\n";
		cerr << "   (must be a positive integer)\n";
		return 1;
	}

	if(base > 16 && argc < 4) {
		cerr << argv[0] << " ERROR: for a base > 16, an alphabet must be specified!\n";
		return 1;
	}
	if(argc > 3)
		alphabet = argv[3];
	if(alphabet.size() < base) {
		cerr << argv[0] << " ERROR: the alphabet has to be as big as the base\n";
		return 1;
	}
	for(int i = 0; i < alphabet.size(); i++) {
		for(int j = i+1; j < alphabet.size(); j++) {
			if(alphabet[i] == alphabet[j]) {
				cerr << argv[0] << " ERROR: the alphabet may not contain the same character multiple times\n";
				return 1;
			}
		}
	}


	cout << "\\begin{picture}("<<diameter+2*offset<<","<<diameter+2*offset<<")(0,0)\n";
	for(int i = 0; i < num; i++) {
		cout << "\t\\node" <<(i==0?"[Nmarks=ir]":"")<<"(q"<<i<<")("
		     << (int(cos(i*2*M_PI/num + M_PI/2)*diameter)+diameter)/2 + offset<< ","
		     << (int(sin(i*2*M_PI/num + M_PI/2)*diameter)+diameter)/2 + offset<< "){"<<i<<"}\n";
	}
	cout << "\n";

	for(int i = 0; i < num; i++) {
		vector<string> transitions(num);
		for(int j = 0; j < base; j++) {
			int succ = ( i * base + j) % num;
			if(transitions[succ] == "") {
				transitions[succ] = string(1,alphabet[j]);
			} else {
				transitions[succ] += ","+string(1,alphabet[j]);
			}
		}
		for(int j = 0; j < num; j++) {
			if(transitions[j] != "") {
				if( i != j ) {
					cout << "\t\\drawedge[curvedepth=4.0](q"<<i<<",q"<<j<<"){"<<transitions[j]<<"}\n";
				} else {
					cout << "\t\\drawloop(q"<<i<<"){"<<transitions[i]<<"}\n";
				}
			}
		}
	}
	cout << "\\end{picture}\n\n";

	return 0;
}

